@@ -1,7 +1,7 @@
 /* crypto.c: crytography-related functions
-   Copyright (c) 2002 Philip Kendall
+   Copyright (c) 2002-2003 Philip Kendall
 
-   $Id: crypto.c,v 1.4 2003/08/22 14:37:33 pak21 Exp $
+   $Id: crypto.c,v 1.8 2003/10/06 12:55:12 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -43,18 +43,19 @@ static const char *signature_format = "(sig-val (dsa (r %m) (s %m)))";
 #define MPI_COUNT 5
 
 static libspectrum_error
-get_signature( GcryMPI *r, GcryMPI *s, libspectrum_byte *data,
+get_signature( gcry_mpi_t *r, gcry_mpi_t *s, libspectrum_byte *data,
 	       size_t data_length, libspectrum_rzx_dsa_key *key );
 static libspectrum_error
-get_hash( GcrySexp *hash, const libspectrum_byte *data, size_t data_length );
+get_hash( gcry_sexp_t *hash, const libspectrum_byte *data,
+	  size_t data_length );
 static libspectrum_error
-create_key( GcrySexp *s_key, libspectrum_rzx_dsa_key *key, const char *format);
-static void free_mpis( GcryMPI *mpis, size_t n );
-static libspectrum_error get_mpi( GcryMPI *mpi, GcrySexp signature,
+create_key( gcry_sexp_t *s_key, libspectrum_rzx_dsa_key *key, int secret_key);
+static void free_mpis( gcry_mpi_t *mpis, size_t n );
+static libspectrum_error get_mpi( gcry_mpi_t *mpi, gcry_sexp_t signature,
 				  const char *token );
 static libspectrum_error
 serialise_mpis( libspectrum_byte **signature, size_t *signature_length,
-		GcryMPI r, GcryMPI s );
+		gcry_mpi_t r, gcry_mpi_t s );
 
 libspectrum_error
 libspectrum_sign_data( libspectrum_byte **signature, size_t *signature_length,
@@ -62,7 +63,7 @@ libspectrum_sign_data( libspectrum_byte **signature, size_t *signature_length,
 		       libspectrum_rzx_dsa_key *key )
 {
   int error;
-  GcryMPI r, s;
+  gcry_mpi_t r, s;
 
   error = get_signature( &r, &s, data, data_length, key );
   if( error ) return error;
@@ -76,22 +77,23 @@ libspectrum_sign_data( libspectrum_byte **signature, size_t *signature_length,
 }
 
 static libspectrum_error
-get_signature( GcryMPI *r, GcryMPI *s, libspectrum_byte *data,
+get_signature( gcry_mpi_t *r, gcry_mpi_t *s, libspectrum_byte *data,
 	       size_t data_length, libspectrum_rzx_dsa_key *key )
 {
-  int error;
-  GcrySexp hash, s_key, s_signature;
+  libspectrum_error error;
+  gcry_error_t gcrypt_error;
+  gcry_sexp_t hash, s_key, s_signature;
 
   error = get_hash( &hash, data, data_length ); if( error ) return error;
 
-  error = create_key( &s_key, key, private_key_format );
+  error = create_key( &s_key, key, 1 );
   if( error ) { gcry_sexp_release( hash ); return error; }
 
-  error = gcry_pk_sign( &s_signature, hash, s_key );
-  if( error ) {
+  gcrypt_error = gcry_pk_sign( &s_signature, hash, s_key );
+  if( gcrypt_error ) {
     libspectrum_print_error( LIBSPECTRUM_ERROR_LOGIC,
 			     "get_signature: error signing data: %s",
-			     gcry_strerror( error ) );
+			     gcry_strerror( gcrypt_error ) );
     gcry_sexp_release( s_key ); gcry_sexp_release( hash );
     return LIBSPECTRUM_ERROR_LOGIC;
   }
@@ -112,11 +114,11 @@ get_signature( GcryMPI *r, GcryMPI *s, libspectrum_byte *data,
 }
 
 static libspectrum_error
-get_hash( GcrySexp *hash, const libspectrum_byte *data, size_t data_length )
+get_hash( gcry_sexp_t *hash, const libspectrum_byte *data, size_t data_length )
 {
-  int error;
+  gcry_error_t error;
   char *digest; size_t digest_length;
-  GcryMPI hash_mpi;
+  gcry_mpi_t hash_mpi;
   
   digest_length = gcry_md_get_algo_dlen( HASH_ALGORITHM );
   digest = malloc( digest_length );
@@ -128,7 +130,8 @@ get_hash( GcrySexp *hash, const libspectrum_byte *data, size_t data_length )
 
   gcry_md_hash_buffer( HASH_ALGORITHM, digest, data, data_length );
 
-  error = gcry_mpi_scan( &hash_mpi, GCRYMPI_FMT_USG, digest, &digest_length );
+  error = gcry_mpi_scan( &hash_mpi, GCRYMPI_FMT_USG, digest, digest_length,
+			 NULL );
   if( error ) {
     libspectrum_print_error( LIBSPECTRUM_ERROR_LOGIC,
 			     "get_hash: error creating hash MPI: %s",
@@ -156,20 +159,25 @@ get_hash( GcrySexp *hash, const libspectrum_byte *data, size_t data_length )
 }
 
 static libspectrum_error
-create_key( GcrySexp *s_key, libspectrum_rzx_dsa_key *key, const char *format )
+create_key( gcry_sexp_t *s_key, libspectrum_rzx_dsa_key *key,
+	    int secret_key )
 {
-  int error;
+  gcry_error_t error;
   size_t i;
-  GcryMPI mpis[MPI_COUNT];
+  gcry_mpi_t mpis[MPI_COUNT];
+  const char *format;
 
   for( i=0; i<MPI_COUNT; i++ ) mpis[i] = NULL;
 
-               error = gcry_mpi_scan( &mpis[0], GCRYMPI_FMT_HEX, key->p, NULL);
-  if( !error ) error = gcry_mpi_scan( &mpis[1], GCRYMPI_FMT_HEX, key->q, NULL);
-  if( !error ) error = gcry_mpi_scan( &mpis[2], GCRYMPI_FMT_HEX, key->g, NULL);
-  if( !error ) error = gcry_mpi_scan( &mpis[3], GCRYMPI_FMT_HEX, key->y, NULL);
-  if( !error && key->x )
-               error = gcry_mpi_scan( &mpis[4], GCRYMPI_FMT_HEX, key->x, NULL);
+  error = gcry_mpi_scan( &mpis[0], GCRYMPI_FMT_HEX, key->p, 0, NULL );
+  if( !error ) 
+    error = gcry_mpi_scan( &mpis[1], GCRYMPI_FMT_HEX, key->q, 0, NULL );
+  if( !error )
+    error = gcry_mpi_scan( &mpis[2], GCRYMPI_FMT_HEX, key->g, 0, NULL );
+  if( !error )
+    error = gcry_mpi_scan( &mpis[3], GCRYMPI_FMT_HEX, key->y, 0, NULL );
+  if( !error && secret_key )
+    error = gcry_mpi_scan( &mpis[4], GCRYMPI_FMT_HEX, key->x, 0, NULL );
 
   if( error ) {
     libspectrum_print_error( LIBSPECTRUM_ERROR_LOGIC,
@@ -178,6 +186,8 @@ create_key( GcrySexp *s_key, libspectrum_rzx_dsa_key *key, const char *format )
     free_mpis( mpis, MPI_COUNT );
     return LIBSPECTRUM_ERROR_LOGIC;
   }
+
+  format = secret_key ? private_key_format : public_key_format;
   
   error = gcry_sexp_build( s_key, NULL, format,
 			   mpis[0], mpis[1], mpis[2], mpis[3], mpis[4] );
@@ -191,23 +201,32 @@ create_key( GcrySexp *s_key, libspectrum_rzx_dsa_key *key, const char *format )
 
   free_mpis( mpis, MPI_COUNT );
 
-  /* FIXME: Use gcry_pk_testkey once it is able to check public keys
-     as well as private keys */
+  /* FIXME: Test public keys as well once gcry_pk_testkey acquires this
+     functionality */
+  if( secret_key ) {
+    error = gcry_pk_testkey( *s_key );
+    if( error ) {
+      libspectrum_print_error( LIBSPECTRUM_ERROR_LOGIC,
+			       "create_key: key is not sane: %s",
+			       gcry_strerror( error ) );
+      return LIBSPECTRUM_ERROR_LOGIC;
+    }
+  }
 
   return LIBSPECTRUM_ERROR_NONE;
 }
 
 static void
-free_mpis( GcryMPI *mpis, size_t n )
+free_mpis( gcry_mpi_t *mpis, size_t n )
 {
   size_t i;
   for( i=0; i<n; i++ ) if( mpis[i] ) gcry_mpi_release( mpis[i] );
 }
 
 static libspectrum_error
-get_mpi( GcryMPI *mpi, GcrySexp sexp, const char *token )
+get_mpi( gcry_mpi_t *mpi, gcry_sexp_t sexp, const char *token )
 {
-  GcrySexp pair;
+  gcry_sexp_t pair;
 
   pair = gcry_sexp_find_token( sexp, token, strlen( token ) );
   if( !pair ) {
@@ -228,13 +247,13 @@ get_mpi( GcryMPI *mpi, GcrySexp sexp, const char *token )
 
 static libspectrum_error
 serialise_mpis( libspectrum_byte **signature, size_t *signature_length,
-		GcryMPI r, GcryMPI s )
+		gcry_mpi_t r, gcry_mpi_t s )
 {
-  int error;
+  gcry_error_t error;
   size_t length, length_s;
   char *ptr;
 
-  error = gcry_mpi_print( GCRYMPI_FMT_PGP, NULL, &length, r );
+  error = gcry_mpi_print( GCRYMPI_FMT_PGP, NULL, 0, &length, r );
   if( error ) {
     libspectrum_print_error( LIBSPECTRUM_ERROR_LOGIC,
 			     "serialise_mpis: length of r: %s",
@@ -242,7 +261,7 @@ serialise_mpis( libspectrum_byte **signature, size_t *signature_length,
     return LIBSPECTRUM_ERROR_LOGIC;
   }
 
-  error = gcry_mpi_print( GCRYMPI_FMT_PGP, NULL, &length_s, s );
+  error = gcry_mpi_print( GCRYMPI_FMT_PGP, NULL, 0, &length_s, s );
   if( error ) {
     libspectrum_print_error( LIBSPECTRUM_ERROR_LOGIC,
 			     "serialise_mpis: length of s: %s",
@@ -259,7 +278,7 @@ serialise_mpis( libspectrum_byte **signature, size_t *signature_length,
     return LIBSPECTRUM_ERROR_MEMORY;
   }
 
-  error = gcry_mpi_print( GCRYMPI_FMT_PGP, *signature, &length, r );
+  error = gcry_mpi_print( GCRYMPI_FMT_PGP, *signature, length, &length, r );
   if( error ) {
     libspectrum_print_error( LIBSPECTRUM_ERROR_LOGIC,
 			     "serialise_mpis: printing r: %s",
@@ -269,7 +288,7 @@ serialise_mpis( libspectrum_byte **signature, size_t *signature_length,
   }
 
   ptr = *signature + length; length = *signature_length - length;
-  error = gcry_mpi_print( GCRYMPI_FMT_PGP, ptr, &length, s );
+  error = gcry_mpi_print( GCRYMPI_FMT_PGP, ptr, length, NULL, s );
   if( error ) {
     libspectrum_print_error( LIBSPECTRUM_ERROR_LOGIC,
 			     "serialise_mpis: printing s: %s",
@@ -281,17 +300,20 @@ serialise_mpis( libspectrum_byte **signature, size_t *signature_length,
   return LIBSPECTRUM_ERROR_NONE;
 }
 
+#include <stdio.h>
+
 libspectrum_error
 libspectrum_verify_signature( libspectrum_rzx_signature *signature,
 			      libspectrum_rzx_dsa_key *key )
 {
   libspectrum_error error;
-  GcrySexp hash, key_sexp, signature_sexp;
+  gcry_error_t gcrypt_error;
+  gcry_sexp_t hash, key_sexp, signature_sexp;
 
   error = get_hash( &hash, signature->start, signature->length );
   if( error ) return error;
 
-  error = create_key( &key_sexp, key, public_key_format );
+  error = create_key( &key_sexp, key, 0 );
   if( error ) { gcry_sexp_release( hash ); return error; }
 
   error = gcry_sexp_build( &signature_sexp, NULL, signature_format,
@@ -307,19 +329,19 @@ libspectrum_verify_signature( libspectrum_rzx_signature *signature,
     return LIBSPECTRUM_ERROR_LOGIC;
   }
 
-  error = gcry_pk_verify( signature_sexp, hash, key_sexp );
+  gcrypt_error = gcry_pk_verify( signature_sexp, hash, key_sexp );
 
   gcry_sexp_release( signature_sexp );
   gcry_sexp_release( key_sexp ); gcry_sexp_release( hash );
 
-  if( error ) {
-    if( error == GCRYERR_BAD_SIGNATURE ) {
+  if( gcrypt_error ) {
+    if( gcry_err_code( gcrypt_error ) == GPG_ERR_BAD_SIGNATURE ) {
       return LIBSPECTRUM_ERROR_SIGNATURE;
     } else {
       libspectrum_print_error(
         LIBSPECTRUM_ERROR_LOGIC,
 	"libgcrypt error verifying signature: %s",
-	gcry_strerror( error )
+	gcry_strerror( gcrypt_error )
       );
       return LIBSPECTRUM_ERROR_LOGIC;
     }
