@@ -2,7 +2,7 @@
                 routines, but not by user code
    Copyright (c) 2001-2002 Philip Kendall, Darren Salt
 
-   $Id: internals.h,v 1.15 2003/08/12 14:23:59 pak21 Exp $
+   $Id: internals.h,v 1.17 2003/10/20 10:34:40 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -64,7 +64,13 @@ libspectrum_dword libspectrum_read_dword( const libspectrum_byte **buffer );
 int libspectrum_write_word( libspectrum_byte **buffer, libspectrum_word w );
 int libspectrum_write_dword( libspectrum_byte **buffer, libspectrum_dword d );
 
-/* zlib (de)compression routines */
+/* (de)compression routines */
+
+libspectrum_error
+libspectrum_uncompress_file( unsigned char **new_buffer, size_t *new_length,
+			     char **new_filename, libspectrum_id_t type,
+			     const unsigned char *old_buffer,
+			     size_t old_length, const char *old_filename );
 
 libspectrum_error
 libspectrum_zlib_inflate( const libspectrum_byte *gzptr, size_t gzlength,
@@ -73,6 +79,14 @@ libspectrum_error
 libspectrum_zlib_compress( const libspectrum_byte *data, size_t length,
 			   libspectrum_byte **gzptr, size_t *gzlength );
 
+libspectrum_error
+libspectrum_gzip_inflate( const libspectrum_byte *gzptr, size_t gzlength,
+			  libspectrum_byte **outptr, size_t *outlength );
+
+libspectrum_error
+libspectrum_bzip2_inflate( const libspectrum_byte *bzptr, size_t bzlength,
+			   libspectrum_byte **outptr, size_t *outlength );
+
 /* Convert a 48K memory dump into separate RAM pages */
 
 int libspectrum_split_to_48k_pages( libspectrum_snap *snap,
