@@ -1,7 +1,7 @@
 /* make-sed.c: Generate a sed script to create the libspectrum_* typedefs
    Copyright (c) 2002-2003 Philip Kendall, Darren Salt
 
-   $Id: make-perl.c,v 1.8 2003/08/12 14:23:59 pak21 Exp $
+   $Id: make-perl.c,v 1.11 2003/12/14 12:05:31 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -110,7 +110,7 @@ int main(void)
 
 #ifdef HAVE_LIB_GLIB		/* #ifdef HAVE_LIB_GLIB */
 
-  printf( "  $_ = \"#include <glib.h>\\n\"\n\n" );
+  printf( "  $_ = '';\n\n" );
 
 #else				/* #ifdef HAVE_LIB_GLIB */
 
@@ -141,6 +141,9 @@ int main(void)
 "GSList* g_slist_append		(GSList		*list,\n"
 "				 gpointer	 data);\n"
 "\n"
+"GSList* g_slist_prepend	(GSList		*list,\n"
+"				 gpointer	 data);\n"
+"\n"
 "GSList* g_slist_remove		(GSList		*list,\n"
 "				 gpointer	 data);\n"
 "\n"
@@ -188,7 +191,7 @@ int main(void)
 
 #ifdef HAVE_GCRYPT_H
 
-  printf( "  $_ = \"  /* The DSA signature parameters 'r' and 's' */\\n  GcryMPI r, s;\\n\";\n\n" );
+  printf( "  $_ = \"  /* The DSA signature parameters 'r' and 's' */\\n  gcry_mpi_t r, s;\\n\";\n\n" );
 
 #else
 
