@@ -1,7 +1,7 @@
 /* rzx.c: routines for dealing with .rzx files
    Copyright (c) 2002-2003 Philip Kendall
 
-   $Id: rzx.c,v 1.47.2.2 2003/09/30 11:39:09 pak21 Exp $
+   $Id: rzx.c,v 1.57 2003/11/27 21:45:25 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -29,6 +29,9 @@
 #include <stddef.h>
 #include <stdio.h>
 #include <string.h>
+#ifdef HAVE_STRINGS_H
+#include <strings.h>		/* Needed for strcasecmp() on QNX6 */
+#endif				/* #ifdef HAVE_STRINGS_H */
 
 #ifdef HAVE_GCRYPT_H
 #include <gcrypt.h>
@@ -302,11 +305,34 @@ libspectrum_rzx_instructions( libspectrum_rzx *rzx )
 
 libspectrum_error
 libspectrum_rzx_read( libspectrum_rzx *rzx, libspectrum_snap **snap,
-	              const libspectrum_byte *buffer, const size_t length,
+	              const libspectrum_byte *buffer, size_t length,
 		      libspectrum_rzx_signature *signature )
 {
   libspectrum_error error;
   const libspectrum_byte *ptr, *end;
+  int uncompressed;
+  libspectrum_byte *new_buffer;
+  libspectrum_id_t raw_type;
+  libspectrum_class_t class;
+
+  /* Find out if this file needs decompression */
+  uncompressed = 0; new_buffer = NULL;
+
+  error = libspectrum_identify_file_raw( &raw_type, NULL, buffer, length );
+  if( error ) return error;
+
+  error = libspectrum_identify_class( &class, raw_type );
+  if( error ) return error;
+
+  if( class == LIBSPECTRUM_CLASS_COMPRESSED ) {
+
+    size_t new_length;
+
+    error = libspectrum_uncompress_file( &new_buffer, &new_length, NULL,
+					 raw_type, buffer, length, NULL );
+    buffer = new_buffer; length = new_length;
+    uncompressed = 1;
+  }
 
   ptr = buffer; end = buffer + length;
 
@@ -317,7 +343,7 @@ libspectrum_rzx_read( libspectrum_rzx *rzx, libspectrum_snap **snap,
   if( signature ) signature->start = NULL;
 
   error = rzx_read_header( &ptr, end, signature );
-  if( error != LIBSPECTRUM_ERROR_NONE ) return error;
+  if( error != LIBSPECTRUM_ERROR_NONE ) { free( new_buffer ); return error; }
 
   while( ptr < end ) {
 
@@ -329,27 +355,42 @@ libspectrum_rzx_read( libspectrum_rzx *rzx, libspectrum_snap **snap,
 
     case LIBSPECTRUM_RZX_CREATOR_BLOCK:
       error = rzx_read_creator( &ptr, end );
-      if( error != LIBSPECTRUM_ERROR_NONE ) return error;
+      if( error != LIBSPECTRUM_ERROR_NONE ) {
+	free( new_buffer );
+	return error;
+      }
       break;
       
     case LIBSPECTRUM_RZX_SNAPSHOT_BLOCK:
       error = rzx_read_snapshot( &ptr, end, snap );
-      if( error != LIBSPECTRUM_ERROR_NONE ) return error;
+      if( error != LIBSPECTRUM_ERROR_NONE ) {
+	free( new_buffer );
+	return error;
+      }
       break;
 
     case LIBSPECTRUM_RZX_INPUT_BLOCK:
       error = rzx_read_input( rzx, &ptr, end );
-      if( error != LIBSPECTRUM_ERROR_NONE ) return error;
+      if( error != LIBSPECTRUM_ERROR_NONE ) {
+	free( new_buffer );
+	return error;
+      }
       break;
 
     case LIBSPECTRUM_RZX_SIGN_START_BLOCK:
       error = rzx_read_sign_start( &ptr, end, signature );
-      if( error != LIBSPECTRUM_ERROR_NONE ) return error;
+      if( error != LIBSPECTRUM_ERROR_NONE ) {
+	free( new_buffer );
+	return error;
+      }
       break;
 
     case LIBSPECTRUM_RZX_SIGN_END_BLOCK:
       error = rzx_read_sign_end( &ptr, end, signature );
-      if( error != LIBSPECTRUM_ERROR_NONE ) return error;
+      if( error != LIBSPECTRUM_ERROR_NONE ) {
+	free( new_buffer );
+	return error;
+      }
       break;
 
     default:
@@ -357,10 +398,12 @@ libspectrum_rzx_read( libspectrum_rzx *rzx, libspectrum_snap **snap,
 	LIBSPECTRUM_ERROR_UNKNOWN,
         "libspectrum_rzx_read: unknown RZX block ID 0x%02x", id
       );
+      free( new_buffer );
       return LIBSPECTRUM_ERROR_UNKNOWN;
     }
   }
 
+  free( new_buffer );
   return LIBSPECTRUM_ERROR_NONE;
 }
 
@@ -371,7 +414,7 @@ rzx_read_header( const libspectrum_byte **ptr, const libspectrum_byte *end,
   libspectrum_dword flags;
 
   /* Check the header exists */
-  if( end - (*ptr) < strlen( rzx_signature ) + 6 ) {
+  if( end - (*ptr) < (ptrdiff_t)strlen( rzx_signature ) + 6 ) {
     libspectrum_print_error( LIBSPECTRUM_ERROR_CORRUPT,
 			     "rzx_read_header: not enough data in buffer" );
     return LIBSPECTRUM_ERROR_CORRUPT;
@@ -774,7 +817,7 @@ rzx_read_sign_end( const libspectrum_byte **ptr, const libspectrum_byte *end,
   length = libspectrum_read_dword( ptr ) - 5;
 
   /* Check there's still enough data */
-  if( end - (*ptr) < length ) {
+  if( end - (*ptr) < (ptrdiff_t)length ) {
     libspectrum_print_error( LIBSPECTRUM_ERROR_CORRUPT,
 			     "rzx_read_sign_end: not enough data in buffer" );
     return LIBSPECTRUM_ERROR_CORRUPT;
@@ -796,19 +839,20 @@ rzx_read_sign_end( const libspectrum_byte **ptr, const libspectrum_byte *end,
 
 #ifdef HAVE_GCRYPT_H
   { 
-    int error; size_t mpi_length;
+    gcry_error_t error; size_t mpi_length;
 
-    mpi_length = length;
-    error = gcry_mpi_scan( &signature->r, GCRYMPI_FMT_PGP, *ptr, &mpi_length );
+    error = gcry_mpi_scan( &signature->r, GCRYMPI_FMT_PGP, *ptr, length,
+			   &mpi_length );
     if( error ) {
       libspectrum_print_error( LIBSPECTRUM_ERROR_CORRUPT,
 			       "error reading 'r': %s",
 			       gcry_strerror( error ) );
       return LIBSPECTRUM_ERROR_CORRUPT;
     }
-    (*ptr) += mpi_length; length -= mpi_length; mpi_length = length;
+    (*ptr) += mpi_length; length -= mpi_length;
 
-    error = gcry_mpi_scan( &signature->s, GCRYMPI_FMT_PGP, *ptr, &mpi_length );
+    error = gcry_mpi_scan( &signature->s, GCRYMPI_FMT_PGP, *ptr, length,
+			   &mpi_length );
     if( error ) {
       libspectrum_print_error( LIBSPECTRUM_ERROR_CORRUPT,
 			       "error reading 's': %s",
@@ -883,16 +927,18 @@ rzx_write_header( libspectrum_byte **buffer, libspectrum_byte **ptr,
 
   strcpy( *ptr, rzx_signature ); (*ptr) += strlen( rzx_signature );
   *(*ptr)++ = 0;		/* Major version number */
-  *(*ptr)++ = 13;		/* Minor version number */
 
   /* Flags */
 #ifdef HAVE_GCRYPT_H
+  *(*ptr)++ = sign ? 13 : 12;	/* Minor version number: 12 if we're not
+				   signing, 13 if we are */
   libspectrum_write_dword( ptr, sign ? 0x01 : 0x00 );
 
   /* Store where to start signing data from */
   *sign_offset = *ptr - *buffer;
 
 #else				/* #ifdef HAVE_GCRYPT_H */
+  *(*ptr)++ = 12;		/* Minor version number */
   libspectrum_write_dword( ptr, 0 );
 #endif				/* #ifdef HAVE_GCRYPT_H */
 
@@ -904,21 +950,30 @@ rzx_write_creator( libspectrum_byte **buffer, libspectrum_byte **ptr,
 		   size_t *length, libspectrum_creator *creator )
 {
   libspectrum_error error;
+  size_t custom_length, block_length;
 
-  error = libspectrum_make_room( buffer, 29, ptr, length );
+  custom_length = libspectrum_creator_custom_length( creator );
+  block_length = 29 + custom_length;
+
+  error = libspectrum_make_room( buffer, block_length, ptr, length );
   if( error != LIBSPECTRUM_ERROR_NONE ) {
     libspectrum_print_error( error, "rzx_write_creator: out of memory" );
     return error;
   }
 
   *(*ptr)++ = LIBSPECTRUM_RZX_CREATOR_BLOCK;
-  libspectrum_write_dword( ptr, 29 );	/* Block length */
+  libspectrum_write_dword( ptr, block_length );	/* Block length */
 
   memcpy( *ptr, libspectrum_creator_program( creator ), 20 ); (*ptr) += 20;
 
   libspectrum_write_word( ptr, libspectrum_creator_major( creator ) );
   libspectrum_write_word( ptr, libspectrum_creator_minor( creator ) );
 
+  if( custom_length ) {
+    memcpy( *ptr, libspectrum_creator_custom( creator ), custom_length );
+    (*ptr) += custom_length;
+  }
+
   return LIBSPECTRUM_ERROR_NONE;
 }
 
