@@ -1,7 +1,7 @@
 /* szx.c: Routines for .szx snapshots
    Copyright (c) 1998,2003 Philip Kendall
 
-   $Id: szx.c,v 1.8 2003/08/17 18:56:21 pak21 Exp $
+   $Id: szx.c,v 1.11 2003/11/27 21:45:25 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -30,8 +30,8 @@
 
 #include "internals.h"
 
-const static char *signature = "ZXST";
-const static size_t signature_length = 4;
+static const char *signature = "ZXST";
+static const size_t signature_length = 4;
 
 static libspectrum_error
 read_chunk( libspectrum_snap *snap, libspectrum_word version,
@@ -76,9 +76,9 @@ write_chunk_header( libspectrum_byte **buffer, libspectrum_byte **ptr,
 		    libspectrum_dword block_length );
 
 static libspectrum_error
-read_ay_chunk( libspectrum_snap *snap, libspectrum_word version,
-	       const libspectrum_byte **buffer, const libspectrum_byte *end,
-	       size_t data_length )
+read_ay_chunk( libspectrum_snap *snap, libspectrum_word version GCC_UNUSED,
+	       const libspectrum_byte **buffer,
+	       const libspectrum_byte *end GCC_UNUSED, size_t data_length )
 {
   size_t i;
 
@@ -101,9 +101,9 @@ read_ay_chunk( libspectrum_snap *snap, libspectrum_word version,
 }
 
 static libspectrum_error
-read_ramp_chunk( libspectrum_snap *snap, libspectrum_word version,
-		 const libspectrum_byte **buffer, const libspectrum_byte *end,
-		 size_t data_length )
+read_ramp_chunk( libspectrum_snap *snap, libspectrum_word version GCC_UNUSED,
+		 const libspectrum_byte **buffer,
+		 const libspectrum_byte *end GCC_UNUSED, size_t data_length )
 {
   libspectrum_word flags;
   libspectrum_byte page, *buffer2;
@@ -169,9 +169,9 @@ read_ramp_chunk( libspectrum_snap *snap, libspectrum_word version,
 }
 
 static libspectrum_error
-read_scld_chunk( libspectrum_snap *snap, libspectrum_word version,
-	       const libspectrum_byte **buffer, const libspectrum_byte *end,
-	       size_t data_length )
+read_scld_chunk( libspectrum_snap *snap, libspectrum_word version GCC_UNUSED,
+		 const libspectrum_byte **buffer,
+		 const libspectrum_byte *end GCC_UNUSED, size_t data_length )
 {
   if( data_length != 2 ) {
     libspectrum_print_error( LIBSPECTRUM_ERROR_UNKNOWN,
@@ -188,8 +188,8 @@ read_scld_chunk( libspectrum_snap *snap, libspectrum_word version,
 
 static libspectrum_error
 read_spcr_chunk( libspectrum_snap *snap, libspectrum_word version,
-		 const libspectrum_byte **buffer, const libspectrum_byte *end,
-		 size_t data_length )
+		 const libspectrum_byte **buffer,
+		 const libspectrum_byte *end GCC_UNUSED, size_t data_length )
 {
   libspectrum_byte out_ula;
 
@@ -218,8 +218,8 @@ read_spcr_chunk( libspectrum_snap *snap, libspectrum_word version,
 
 static libspectrum_error
 read_z80r_chunk( libspectrum_snap *snap, libspectrum_word version,
-		 const libspectrum_byte **buffer, const libspectrum_byte *end,
-		 size_t data_length )
+		 const libspectrum_byte **buffer,
+		 const libspectrum_byte *end GCC_UNUSED, size_t data_length )
 {
   if( data_length != 37 ) {
     libspectrum_print_error( LIBSPECTRUM_ERROR_UNKNOWN,
@@ -271,8 +271,10 @@ read_z80r_chunk( libspectrum_snap *snap, libspectrum_word version,
 }
 
 static libspectrum_error
-skip_chunk( libspectrum_snap *snap, libspectrum_word version,
-	    const libspectrum_byte **buffer, const libspectrum_byte *end,
+skip_chunk( libspectrum_snap *snap GCC_UNUSED,
+	    libspectrum_word version GCC_UNUSED,
+	    const libspectrum_byte **buffer,
+	    const libspectrum_byte *end GCC_UNUSED,
 	    size_t data_length )
 {
   *buffer += data_length;
@@ -281,7 +283,7 @@ skip_chunk( libspectrum_snap *snap, libspectrum_word version,
 
 struct read_chunk_t {
 
-  char *id;
+  const char *id;
   read_chunk_fn function;
 
 };
@@ -574,14 +576,23 @@ write_crtr_chunk( libspectrum_byte **buffer, libspectrum_byte **ptr,
 		  size_t *length, libspectrum_creator *creator )
 {
   libspectrum_error error;
+  size_t custom_length;
 
-  error = write_chunk_header( buffer, ptr, length, "CRTR", 36 );
+  custom_length = libspectrum_creator_custom_length( creator );
+
+  error = write_chunk_header( buffer, ptr, length, "CRTR",
+			      36 + custom_length );
   if( error ) return error;
 
   memcpy( *ptr, libspectrum_creator_program( creator ), 32 ); *ptr += 32;
   libspectrum_write_word( ptr, libspectrum_creator_major( creator ) );
   libspectrum_write_word( ptr, libspectrum_creator_minor( creator ) );
 
+  if( custom_length ) {
+    memcpy( *ptr, libspectrum_creator_custom( creator ), custom_length );
+    *ptr += custom_length;
+  }
+
   return LIBSPECTRUM_ERROR_NONE;
 }
 
@@ -590,7 +601,7 @@ write_z80r_chunk( libspectrum_byte **buffer, libspectrum_byte **ptr,
 		  size_t *length, libspectrum_snap *snap )
 {
   libspectrum_dword tstates;
-
+  libspectrum_byte flags;
   libspectrum_error error;
 
   error = write_chunk_header( buffer, ptr, length, "Z80R", 37 );
@@ -630,8 +641,10 @@ write_z80r_chunk( libspectrum_byte **buffer, libspectrum_byte **ptr,
     *(*ptr)++ = '\0';
   }
 
-  /* Flags; 'last instruction EI' flag not supported, but 'halted' is */
-  *(*ptr)++ = libspectrum_snap_halted( snap ) ? 0x02 : 0x00;
+  flags = '\0';
+  if( libspectrum_snap_last_instruction_ei( snap ) ) flags |= 0x01;
+  if( libspectrum_snap_halted( snap ) ) flags |= 0x02;
+  *(*ptr)++ = flags;
 
   /* Hidden register not supported */
   *(*ptr)++ = '\0';
