@@ -1,7 +1,7 @@
 /* timings.c: Timing routines
    Copyright (c) 2003 Philip Kendall
 
-   $Id: timings.c,v 1.6 2003/05/25 09:33:52 pak21 Exp $
+   $Id: timings.c,v 1.7 2003/11/27 21:45:25 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -67,7 +67,7 @@ static timings_t base_timings[] = {
   { 3546900, 1773400, 36, 128, 28, 32, 64, 192, 48, 16, 17988 }, /* Pentagon */
   { 3546900, 1773400, 24, 128, 24, 52, 48, 192, 48, 23, 14361 }, /* +2A */
   { 3546900, 1773400, 24, 128, 24, 52, 48, 192, 48, 23, 14361 }, /* +3 */
-  { 0 },					          /* Unknown machine */
+  { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },			  /* Unknown machine */
   { 3500000,       0, 24, 128, 24, 48, 48, 192, 48, 24, 14336 }, /* 16K */
   { 3528000, 1764750, 24, 128, 24, 48, 48, 192, 48, 27, 14336 }, /* TC2068 */
 };
