@@ -1,7 +1,7 @@
 /* tzx_read.c: Routines for reading .tzx files
    Copyright (c) 2001, 2002 Philip Kendall, Darren Salt
 
-   $Id: tzx_read.c,v 1.51 2003/07/19 09:17:13 pak21 Exp $
+   $Id: tzx_read.c,v 1.52.2.1 2004/02/11 21:20:49 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -403,7 +403,7 @@ tzx_read_pulses_block( libspectrum_tape *tape, const libspectrum_byte **ptr,
   libspectrum_tape_block_set_count( block, count );
 
   /* Check enough data exists for every pulse */
-  if( end - (*ptr) < 2 * count ) {
+  if( end - (*ptr) < (ptrdiff_t)( 2 * count ) ) {
     free( block );
     libspectrum_print_error(
       LIBSPECTRUM_ERROR_CORRUPT,
@@ -688,7 +688,7 @@ tzx_read_select( libspectrum_tape *tape, const libspectrum_byte **ptr,
 
   /* Get the number of selections */
   count = **ptr; (*ptr)++;
-  libspectrum_tape_block_set_count( block, count ); (*ptr)++;
+  libspectrum_tape_block_set_count( block, count );
 
   /* Allocate memory */
   offsets = malloc( count * sizeof( int ) );
