@@ -1,7 +1,7 @@
 /* z80.c: Routines for handling .z80 snapshots
    Copyright (c) 2001-2002 Philip Kendall, Darren Salt
 
-   $Id: z80.c,v 1.61 2003/09/12 12:47:04 pak21 Exp $
+   $Id: z80.c,v 1.63 2003/11/07 14:06:19 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -499,6 +499,7 @@ read_slt( libspectrum_snap *snap, const libspectrum_byte **next_block,
 	return LIBSPECTRUM_ERROR_CORRUPT;
       }
 
+      length = 0;	/* Tell uncompress_block to allocate memory for us */
       error =
 	uncompress_block( &buffer, &length,
 			  *next_block + offsets[i], slt_length[i] );
@@ -811,7 +812,9 @@ libspectrum_z80_write2( libspectrum_byte **buffer, size_t *length,
 
   *out_flags = 0;
 
-  /* .z80 format doesn't store the 'halted' state */
+  /* .z80 format doesn't store the 'last instruction EI' or 'halted' state */
+  if( libspectrum_snap_last_instruction_ei( snap ) ) 
+    *out_flags |= LIBSPECTRUM_FLAG_SNAPSHOT_MINOR_INFO_LOSS;
   if( libspectrum_snap_halted( snap ) )
     *out_flags |= LIBSPECTRUM_FLAG_SNAPSHOT_MINOR_INFO_LOSS;
 
